require_relative 'player'
require_relative 'die'
require_relative 'gameturn'
require_relative 'treasure_trove'

module StudioGame
  class Game

    #attributes
    attr_reader :title

    def initialize (title)
      @title = title.capitalize
      @players=[]
    end

    def addplayer (a_player)
      @players << a_player
      #same as @players.push(a_player)
    end

    def load_players(from_file)
      File.readlines(from_file).each do |line|
        addplayer(Player.from_csv(line))
      end
    end

    def save_high_scores(to_file="high_scores.txt")
      File.open(to_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        @players.sort.each do |player|
          formatted_name = player.name.ljust(20, '.')
          file.puts "#{formatted_name} #{player.score}"
        end
      end
    end

    def total_points
      @players.reduce(0) { |sum, player| sum + player.points }
    end

    def play(gamenumber)
      puts "There are #{@players.size} players in #{@title}:"
      @players.each do |player|
        puts player
      end

      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures to be found:"
      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points"
      end

      1.upto(gamenumber) do |count|
        puts "\nGame Number #{count}:"
          @players.each do |player|
          GameTurn.take_turn(player)
          end
        end
      end

      def print_name_and_health(player)
        puts "#{player.name} (#{player.health})"
      end

      def high_score_entry(player)
      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
      end

    def print_stats
      strong_players = @players.select {|player| player.strong?}
      wimpy_players = @players.reject {|player| player.strong?}
      sorted_players = @players.sort { |a,b| b.score <=> a.score}
      #or
      #strong_players, wimpy_players = @players.partition {|player| player.strong?}
      puts "\n#{@title} Statistics:"
      puts "\n#{strong_players.size} strong players:"

      #strong_players
      strong_players.each do |player|
        print_name_and_health(player)
      end

      #wimpy_players
      puts "\n#{wimpy_players.size} wimpy players:"
      wimpy_players.each do |player|
        print_name_and_health(player)
      end

      #high scores
      puts "\n#{@title} High Scores:"
      @players.sort.each do |player|
        puts high_score_entry(player)
      end

        puts "\n#{total_points} total points from treasures found"

      # @players.each do |player|
      #   puts "\n#{player.name}'s point totals:"
      #   puts "#{player.points} grand total points"
      # end

      @players.sort.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.points} grand total points"
      end
    end
  end
end
